# vim: set fileencoding=utf-8 :
"""
Truncate letters filter works similar as 
truncatewords filter of builtin templatetags


Usage:

    {{ value|truncateletters:num }}
    {{ value|truncateletters_html:num }}



AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""   
from __future__ import with_statement
from django import template
from django.utils.encoding import force_unicode
from django.utils.functional import allow_lazy
from django.template.defaultfilters import stringfilter
from django.utils.safestring import mark_safe

import re

register = template.Library()

def truncate_letters(s, num):
    "Truncates a string after a certain number of letters."
    if not s or s == "":
        return u""
    
    s = force_unicode(s)
    length = int(num)
    counter = 0
    for last, word in enumerate(s):
        if word in ('\n', '\r', '\s', '\t'): continue
        counter += 1
        if counter == length: break
    return s[:last+1] + (u'...' if last+1 < len(s) else '')
truncate_letters = allow_lazy(truncate_letters, unicode)

@register.filter
@stringfilter
def truncateletters(value, arg):
    """
    Truncates a string after a certain number of letters.

    Argument: Number of letters to truncate after.
    """
    try:
        length = int(arg)
    except ValueError: # Invalid literal for int().
        return value # Fail silently.
    return truncate_letters(value, length)
truncateletters.is_safe = True

def _find(value, start, last):
    last = value.find('>', start)
    last += 1
    return value[start:last]

def _isend(tag):
    r = re.compile("(?:<\s*/\s*[^>]*>)|(?:<[^/>]*/\s*>)")
    return not r.match(tag) is None

def _name(tag):
    r = re.compile("<\s*([^\s>]*)\s*.*?>")
    m = r.match(tag)
    return m.group(1) if m is not None else None

def truncate_html_letters(s, num):
    """
    Truncates html to a certain number of letters (not counting tags and
    comments). Closes opened tags if they were correctly closed in the given
    html.
    """
    if not s or s == "":
        return u""
    
    s = force_unicode(s)
    length = int(num)
    if length <= 0:
        return u''
    counter = 0
    skip = 0
    start_tags = []
    end_tags = []
    for last, word in enumerate(s):
        if skip > 0:
            skip -= 1
            continue
        elif word == '<':
            tag = _find(s, last, last)
            skip = len(tag) - 1
            if _isend(tag):
                end_tags.append(tag.translate(' \t'))
            else:
                start_tags.append(tag)
            continue
        elif word in ('\n', '\r', '\s', '\t'):
            continue
        counter += 1
        if counter == length:
            break
    suffix = ''
    for start_tag in start_tags:
        end_tag = "</%s>"%_name(start_tag)
        if not end_tag in end_tags:
            suffix = end_tag + suffix
        else:
            end_tags.remove(end_tag)
    suffix = '...' + suffix
    return mark_safe(s[:last+1].strip() + u'%s'%(suffix if last+1 < len(s) else ''))
truncate_html_letters = allow_lazy(truncate_html_letters, unicode)

@register.filter
@stringfilter
def truncateletters_html(value, arg):
    """
    Truncates HTML after a certain number of letters.

    Argument: Number of letters to truncate after.
    """
    try:
        length = int(arg)
    except ValueError: # invalid literal for int()
        return value # Fail silently.
    return truncate_html_letters(value, length)
truncateletters_html.is_safe = True



# vim: set fileencoding=utf-8 :
"""
Middleware to store request instance during request


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local

__all__ = ['get_request', 'ThreadLocalsMiddleware']
_thread_locals = local()

def get_request():
    """Return stored request instance in current thread"""
    return getattr(_thread_locals, 'request', None)

class ThreadLocalsMiddleware(object):
    """
    Middleware that store current request in thread local storage.
    This middleware should come at the top of the MIDDLEWARE_CLASSES list.
    
    """
    def process_request(self, request):
        # save current request instance
        _thread_locals.request = request

    def process_response(self, request, response):
        # remove saved request instance
        if hasattr(_thread_locals, 'request'):
            delattr(_thread_locals, 'request')
        return response



# vim: set fileencoding=utf-8 :
"""
Decorator of ``Form``/``View`` for handling 
``request`` with ``Form`` instance

Usage::

    # --- forms.py
    from django import forms
    from qwert.decorators import with_request

    from models import SomeModel

    @with_request
    class SomeModelForm(forms.ModelForm):
        class Meta:
            model = SomeModel
        
        def save(self, commit=True):
            instance = super(SomeModel, self).save(False)
            instance.author = self.request.user
            if commit:
                instance.save()
            return instance

    # --- views.py
    from django.views.generic import CreateView
    from qwert.decorators import with_request

    from models import SomeModel
    from forms import SomeModelForm

    @with_request
    class SomeModelCreateView(CreateView):
        model = SomeModel
        form_class = SomeModelForm


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from django.forms.models import ModelForm
from django.views.generic.edit import FormMixin


def _with_request_form(form_class):
    """
    decorate form_class to store ``request`` 
    constrcutor argument with the instance

    """
    if not getattr(form_class, '_with_request_decorated', False):
        # store original constructor
        original_constructor = form_class.__init__

        def __init__(self, request, *args, **kwargs):
            self.request = request
            # call original constructor
            original_constructor(self, *args, **kwargs)

        # override constructor
        form_class.__init__ = __init__
        form_class._with_request_decorated = True
    return form_class

def _with_request_view(view_class):
    """
    decorate view_class to pass ``request`` to construtor
    of a form as kwargs

    """
    if not getattr(view_class, '_with_request_decorated', False):
        # store original get_form_kwargs
        original_get_form_kwargs = view_class.get_form_kwargs

        def get_form_kwargs(self):
            # call original get_form_kwargs
            kwargs = original_get_form_kwargs(self)
            kwargs['request'] = self.request
            return kwargs

        # override get_form_kwargs
        view_class.get_form_kwargs = get_form_kwargs
        view_class._with_request_decorated = True
    return view_class

def with_request(cls):
    """A class decorator for handling request in form instance

    Usage::

        # --- forms.py
        from django import forms
        from qwert.decorators import with_request

        from models import SomeModel

        @with_request
        class SomeModelForm(forms.ModelForm):
            class Meta:
                model = SomeModel
            
            def save(self, commit=True):
                instance = super(SomeModel, self).save(False)
                instance.author = self.request.user
                if commit:
                    instance.save()
                return instance

        # --- views.py
        from django.views.generic import CreateView
        from qwert.decorators import with_request

        from models import SomeModel
        from forms import SomeModelForm

        @with_request
        class SomeModelCreateView(CreateView):
            model = SomeModel
            form_class = SomeModelForm

    """
    if issubclass(cls, ModelForm):
        return _with_request_form(cls)
    elif issubclass(cls, FormMixin):
        return _with_request_view(cls)
    else:
        raise AttributeError(
                "%r must be a subclass of ModelForm or FormMixin" % cls
            )



# vim: set fileencoding=utf-8 :
from django import template
from django.conf import settings

register = template.Library()

@register.filter("urlize_html")
def urlize_html(html):
    """
    Returns urls found in an (X)HTML text node element as urls via Django urlize filter.
    """
    try:
        from BeautifulSoup import BeautifulSoup
        from django.utils.html import urlize
    except ImportError:
        if settings.DEBUG:
            raise template.TemplateSyntaxError(
                    "Error in urlize_html The Python BeautifulSoup libraries aren't installed."
                )
        return html
    else:
        soup = BeautifulSoup(html)
           
        textNodes = soup.findAll(text=True)
        for textNode in textNodes:
            urlizedText = urlize(textNode)
            textNode.replaceWith(urlizedText)
            
        return str(soup)



class Http403(Exception):
    """
    An exception similar as ``django.http.Http404`` but for ``HttpResponseForbidden``
    ``qwert.middleware.Http403Middleware`` is required to handle this exception.

    """
    pass



# vim: set fileencoding=utf-8 :
# http://www.freewisdom.org/projects/python-markdown/Django
from django import template
from django.template.defaultfilters import stringfilter
from django.utils.safestring import mark_safe

from django.conf import settings

register = template.Library()

@register.filter(name='markdown')
@stringfilter
def markdown_filter(value, arg=''):
    """
    Filter to create HTML out of Markdown, using custom extensions.

    The diffrence between this filter and the django-internal markdown
    filter (located in ``django/contrib/markup/templatetags/markup.py``)
    is that this filter enables extensions to be load.

    Usage::

        {{ object.text|markdown }}
        {{ object.text|markdown:"save" }}
        {{ object.text|markdown:"codehilite" }}
        {{ object.text|markdown:"save,codehilite" }}

    This code is taken from
    http://www.freewisdom.org/projects/python-markdown/Django
    """
    try:
        import markdown
    except ImportError:
        if settings.DEBUG:
            raise template.TemplateSyntaxError(
                   "Error in {% markdown %} filter: "
                   "The markdown library isn't installed."
                )
        else :
            from django.utils.html import escape, linebreaks
            return linebreaks(escape(value))
    else:
        extensions=arg.split(",")
        if len(extensions) > 0 and extensions[0] == "safe" :
            extensions = extensions[1:]
            safe_mode = True
        else :
            safe_mode = False
        return mark_safe(markdown.markdown(value, extensions, safe_mode=safe_mode))



# vim: set fileencoding=utf-8 :
# from snippets: http://djangosnippets.org/snippets/45/
from django import template
from django.conf import settings

register = template.Library()

@register.filter
def get_links(value):
    """
    Returns links found in an (X)HTML string as Python objects for itteration in templates.
    
    EXAMPLE:
    
    <ul>
      {% for link in blog.entry.body|get_links %}
         <li><a href="{{ link.href }}">{{ link.title }}</a></li>
      {% endfor %}
    </ul>
    
    """
    try:
        import beautifulsoup
    except ImportError:
        if settings.DEBUG:
            raise template.TemplateSyntaxError, "Error in {% getlinks %} filter: The Python BeautifulSoup and/or urllib2 libraries aren't installed."
        return value
    else:
        soup = beautifulsoup.BeautifulSoup(value)
        return soup.findAll('a')



# vim: set fileencoding=utf-8 :
# from snippets: http://djangosnippets.org/snippets/935/
import sys
from django.views.debug import technical_500_response


class UserBasedExceptionMiddleware(object):
    """User based exception middleware
    
    If request user is superuser then display DEBUG information for 500 error
    """
    def process_exception(self, request, exception):
        if request.user.is_superuser:
            return technical_500_response(request, *sys.exc_info())
        return None



# vim: set fileencoding=utf-8 :
# Ref: http://djangosnippets.org/snippets/9/
from django import template
from django.utils.translation import gettext_lazy as _
import re

register = template.Library()

"""
This tag can be used to calculate a python expression, and save it into a template variable which you can reuse 
later or directly output to template. So if the default django tag can not be suit for your need, you can use it.

How to use it

{% expr "1" as var1 %}
{% expr [0, 1, 2] as var2 %}
{% expr _('Menu') as var3 %}
{% expr var1 + "abc" as var4 %}
...
{{ var1 }}

for 0.2 version

{% expr 3 %}
{% expr "".join(["a", "b", "c"]) %}

Will directly output the result to template

Syntax

{% expr python_expression as variable_name %}

python_expression can be valid python expression, and you can even use _() to translate a string. Expr tag also can used context variables. 
"""

class ExprNode(template.Node):
    def __init__(self, expr_string, var_name):
        self.expr_string = expr_string
        self.var_name = var_name
    
    def render(self, context):
        try:
            clist = list(context)
            clist.reverse()
            d = {}
            d['_'] = _
            for c in clist:
                d.update(c)
            if self.var_name:
                context[self.var_name] = eval(self.expr_string, d)
                return ''
            else:
                return str(eval(self.expr_string, d))
        except:
            raise

r_expr = re.compile(r'(.*?)\s+as\s+(\w+)', re.DOTALL)    
def do_expr(parser, token):
    try:
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError, "%r tag requires arguments" % token.contents[0]
    m = r_expr.search(arg)
    if m:
        expr_string, var_name = m.groups()
    else:
        if not arg:
            raise template.TemplateSyntaxError, "%r tag at least require one argument" % tag_name
            
        expr_string, var_name = arg, None
    return ExprNode(expr_string, var_name)
do_expr = register.tag('expr', do_expr)



# vim: set fileencoding=utf-8 :
from setuptools import setup, find_packages

version = __import__('qwert').__version__

def read(filename):
    import os.path
    return open(os.path.join(os.path.dirname(__file__), filename)).read()

def readlist(filename):
    import os.path
    app_list = open(os.path.join(os.path.dirname(__file__), filename)).readlines()
    app_list = map((lambda x: None if x.startswith('#') else x.strip()), app_list)
    app_list = filter(bool, app_list)
    return app_list

setup(
    name="django-qwert",
    version=version,
    description = "A trivia collection framework for Django",
    long_description=read('README.rst'),
    classifiers = [
        # http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ],
    keywords = "django trivia snipet collection framework",
    author = "Alisue",
    author_email = "lambdalisue@hashnote.net",
    url=r"https://github.com/lambdalisue/django-qwert",
    download_url = r"https://github.com/lambdalisue/django-qwert/tarball/master",
    license = 'MIT',
    zip_safe=False,
    packages = find_packages(exclude=['tests']),
    include_package_data = True,
    test_suite='runtests.runtests',
    tests_require=readlist('requirements-test.txt'),
    install_requires=readlist('requirements.txt'),
)
